# `go mod`命令

自从 Go 1.13 正式发布 Go Module 以后，基本上绝大多数的 Go 程序都采用了模块化的组织方式。Go Module 的核心概念就是每一个 Go 程序，不管是可执行的二进制程序还是一个可以被其他程序所使用的依赖包，都是以一个模块的形式存在的。

因为 Go 1.0 在发布的时候已经基本上固定了依赖包的获取和缓存等机制，所以 Go Module 也只能在这个机制上进行改进。接下来会有专门的一章来具体的对 Go Module 中一些常见的概念和应用进行说明。这里先简要的介绍一下`go mod`命令提供的功能。

`go mod`命令所提供的功能是通过它引入的另一级的子命令完成的，所以`go mod`命令的使用格式就变成了`go mod <command>`

## `go mod download`

`download`命令的主要目的是把指定的模块下载到本地缓存中，这通常都是为了提高 Go 编译速度而做的提前准备。将依赖包模块下载到本地缓存也有助于 Go 程序的离线编译。

## `go mod init`

`init`命令是一个一般情况下只会用一次的命令，它的功能就是创建一个新的 Go 模块。例如我们可以在一个空白的新目录中执行命令`go mod init .`，这样就可以在这个目录中初始化建立一个新的 Go 模块，其中会包括所需的`go.mod`、`go.sum`和`main.go`文件。

目前基本上每一个 Go 程序都是从`go mod init`命令开始的，它的命令格式为`go mod init <目标模块路径>`。如果指定的目录不存在，那么`go mod init`会自动创建一个新的目录。

## `go mod edit`

基于 Go Module 建立的程序，其模块定义和依赖包都是通过`go.mod`文件定义的，这个文件一般情况下是不建议手工修改的。但是为了有效合理的修改这个文件，`go mod`提供了一个`edit`命令。

`edit`命令的常用格式为`go mod edit [编辑内容] [go.mod位置]`。如果不提供`go.mod`文件的位置，那么`go mod edit`命令将默认修改当前目录中的`go.mod`文件。

以下是`go mod edit`命令所常用的编辑内容参数。

- `-module`，用于更改模块的路径，通常是`go.mod`文件中`module`一行的内容。
- `-require=路径@版本`，用于增加一个指定路径的模块依赖。
- `-droprequire=路径`，用于删除一个指定路径的模块依赖。
- `-exclude=路径@版本`，用于排除一个指定路径模块的依赖，被排除的内容将不会被引入到程序依赖中。注意，如果指定了排除的版本，那么依赖包的其他版本还是可以被加入到程序中的。
- `-dropexclude=路径@版本`，用于解除一个指定路径模块的依赖排除。
- `-replace=旧路径[@版本]=新路径[@版本]`，用于定义一个依赖包的替换，这种替换可以为指定的依赖包提供一个同功能的但路径不同的镜像。
- `-dropreplace=旧路径[@版本]`，用于删除一个依赖包的替换定义。
- `-retract=版本`，用于增加一个版本的撤回提醒。
- `-dropretract=版本`，用于撤回一个版本的撤回提醒。
- `-go=版本`，定义模块所期望使用的 Go 版本。

## `go mod tidy`

`tidy`命令的功能其实非常简单，它所提供的功能就是根据当前模块中代码时间引用的依赖包情况，对`go.mod`文件中列举的额外的依赖包进行清理，`tidy`命令不仅会删除当前程序中没有使用到的依赖包，还会自动添加已经在程序中使用了但是还没有被添加到`go.mod`中的依赖包。

```admonish caution
`go mod tidy`在使用的时候需要额外注意一下，如果我们提前在`go.mod`文件中声明所要依赖的依赖包，但是在程序中还没有使用到的话，此时调用`go mod tidy`命令将会使这个依赖包被清理掉。
```

## `go mod vendor`

`vendor`命令将重置模块的`vendor`目录，并将所有编译需要的依赖包放置在其中。
