# 基本数据类型

Go 中的数据类型主要有布尔、数字、字符串、数组、切片、结构体、指针、函数、接口、Map 和通道。这一章中将主要介绍各个基本类型的语法定义和基本功能，其具体的使用方法将在后文中的独立章节中进行具体说明。

在 Go 中数据类型的语法定义如下：

```
Type  = TypeName [ TypeArgs ] | TypeLit | "(" Type ")" .
TypeName = identifier | QualifiedIdent .
TypeArgs = "[" TypeList [ "," ] "]" .
TypeList = Type { "," Type } .
TypeLit = ArrayType | StructType | PointerType | FunctionType | InterfaceType | SliceType | MapType | ChannelType .
```

在这个语法定义中，`TypeArgs`表示一个类型参数，是后面索要介绍的泛型中使用的，目前可以暂时不必理会。目前需要从语法定义中了解的只有 Go 中常用的数据类型的书写方法。

## 布尔类型

布尔类型是 Go 语言中最简单也是使用最广的类型，其类型标识符为`bool`，可取值为预定义的常量`true`和`false`，分别对应真值和假值。

## 数字类型

数字类型是整型、浮点型、复数型三种类型的统称，它们分别用于表示整数、浮点值和复数值。用于表示数字类型的标识符都是 Go 语言预先定义好的，全局有效的。数字类型主要有以下这些。

| 类型标识符   | 指代类型                     | 长度（位） | 最小值               | 最大值               |
| ------------ | ---------------------------- | ---------- | -------------------- | -------------------- |
| `uint8`      | 无符号 8 位整型              | 8          | 0                    | 255                  |
| `uint16`     | 无符号 16 位整型             | 16         | 0                    | 65535                |
| `uint32`     | 无符号 32 位整型             | 32         | 0                    | 4294967295           |
| `uint64`     | 无符号 64 位整型             | 64         | 0                    | 18446744073709551615 |
| `int8`       | 有符号 8 位整型              | 8          | -128                 | 127                  |
| `int16`      | 有符号 16 位整型             | 16         | -32768               | 32767                |
| `int32`      | 有符号 32 位整型             | 32         | -2147483648          | 2147483647           |
| `int64`      | 有符号 64 位整型             | 64         | -9223372036854775808 | 9223372036854775807  |
| `float32`    | IEEE--754 32 位浮点          | 32         |                      |                      |
| `float64`    | IEEE-754 64 位浮点           | 64         |                      |                      |
| `complex64`  | `float32`型实数和虚数复数    | 64         |                      |                      |
| `complex128` | `float64`型实数和虚数复数    | 128        |                      |                      |
| `byte`       | 单字节整型，`uint8`别名      | 8          |                      |                      |
| `rune`       | 字符型，`int32`别名          | 32         |                      |                      |
| `uint`       | 机器架构长度无符号整型       | 32 或 64   |                      |                      |
| `int`        | 机器架构长度有符号整型       | 32 或 64   |                      |                      |
| `uintptr`    | 足以存放内存地址的无符号整型 |            |                      |                      |

## 字符串类型

字符串类型是一个可能为空的字节序列，是一个字符值的集合。字符串类型使用`string`预定义标识符声明。字符串的内容一旦创建即不可更改，

字符串中的元素可以通过下标访问，其中元素的索引从 0 开始。

## 数组类型

数组类型是由一个单一类型元组组成的序列类型，数组中元素的数量称为数组的长度。数组类型的定义语法格式如下。

```
ArrayType = "[" ArrayLength "]" ElementType .
ArrayLength = Expression .
ElementType = Type .
```

根据数组类型的语法定义，一个数组类型可以使用`[7]int`、`[20]byte`来表示。这里需要注意的是，数组类型中的长度是数组类型的一部分，必须是能够确定的`int`类型非负常数，数组一旦定义，其长度在数组的作用域内就不再可变。数组总是一维的，但是可以通过组合来形成多维数组，例如`[4][5]int`。

数组中的元素可以通过整型索引来访问。

## 切片类型

之前介绍过的数组类型是一个长度固定不可变的序列类型，切片则是一个不固定长度的序列类型。与数组一样，切片也是通过下标去访问其中元素的。切片类型的定义语法如下。

```
SliceType = "[" "]" ElementType .
```

根据切片类型的语法定义，一个切片类型可以使用`[]int`、`[]byte`来表示。与数组类型不同的是，切片类型的类型中没有切片长度的部分。但是与数组一样，切片也是一维的，并且可以通过组合形成多维切片，但是由于切片的长度是可以动态变化的，所以多维切片中的每一个内部切片都需要单独完成初始化。

## 结构体

结构体是一个由多种类型组合在一起形成的一个能够记录和描述复杂内容的类型。其中每一个命名元素都有自己端粒的类型，被称为字段。结构体的具体语法定义将放在后面专门讲述结构体的章节详细说明。

## 指针类型

指针无论在哪个语言中都是一个非常神奇和灵活的类型，它在理论上可以指向任意的内存地址。在 Go 中一个指针只能指向给定类型的变量地址，这个给定的类型被称为指针的基类型。指针类型的语法定义如下。

```
PointerType = "*" BaseType .
BaseType = Type .
```

根据指针类型的语法定义，一个指针类型可以使用`*int`、`*[]byte`的形式来表示。

## Map 类型

Map 类型主要功能是用来保存键值对，是一种无序的序列，由两种类型组成，其中作为值的类型也被称为元素类型，另一种则作为值的索引，也被称为键类型。Map 类型的语法定义如下。

```
MapType = "map" "[" KeyType "]" ElmentType .
KeyType = Type .
```

根据 Map 类型的语法定义，以下 Map 类型的书写都是合法的：`map[string]int`、`map[string]iterface{}`。

## 通道类型

Go 里提供的通道类型提供了一种机制，可以在不同的协程和线程之间传输指定类型的元素来实现通信。用于实现这种通信功能的类型就是通道。通道的语法定义如下。

```
ChannelType = ( "chan" | "chan" "<-" | "<-" "chan" ) ElementType .
```

具体通道类型的使用将在后续的独立章节中介绍。
