# 标识符和关键字

标识符和关键字在任何语言的程序中都是组成程序代码的基础元素，它们在程序代码中都有独特且专一的功能。

## 标识符

标识符在代码中主要用来命名程序实体，例如变量、类型等。在 Go 语言中，标识符是一个由一个或者多个字母或数字组成的序列，其第一个字符必须是字母，不能是数字。

标识符的语法定义为：

```
identifier = letter { letter | unicode_digit } .
```

在 Go 语言中，由一些标识符是在全局隐式定义的，它们已经具有了特殊含义的指代，不可被挪用于指代其他内容。以下表格中即是 Go 在全局定义的标识符。

| 标识符       | 类型 | 含义                                                     |
| ------------ | ---- | -------------------------------------------------------- |
| `any`        | 类型 | 指代任意类型                                             |
| `bool`       | 类型 | 指代布尔类型                                             |
| `byte`       | 类型 | 指代字节类型                                             |
| `comparable` | 类型 | 指代可比较类型                                           |
| `complex64`  | 类型 | 指代 64 位复数类型                                       |
| `complex128` | 类型 | 指代 128 位复数类型                                      |
| `error`      | 类型 | 指代描述错误的接口                                       |
| `float32`    | 类型 | 指代 32 位单精度浮点类型                                 |
| `float64`    | 类型 | 指代 64 位双精度浮点类型                                 |
| `int`        | 类型 | 指代当前机器架构支持长度的整型                           |
| `int8`       | 类型 | 指代 8 位长度的单字节整型                                |
| `int16`      | 类型 | 指代 16 位长度的双字节整型                               |
| `int32`      | 类型 | 指代 32 位长度的四字节整型                               |
| `int64`      | 类型 | 指代 64 位长度的八字节整型                               |
| `rune`       | 类型 | 指代一个 UTF-8 码点字符                                  |
| `string`     | 类型 | 指代字符串类型                                           |
| `uint`       | 类型 | 指代当前机器架构支持长度的无符号整型                     |
| `uint8`      | 类型 | 指代 8 位长度的单字节无符号整型                          |
| `uint16`     | 类型 | 指代 16 位长度的双字节无符号整型                         |
| `uint32`     | 类型 | 指代 32 位长度的四字节无符号整型                         |
| `uint64`     | 类型 | 指代 64 位长度的八字节无符号整型                         |
| `uintptr`    | 类型 | 指代一个用于保存指针地址的整型                           |
| `true`       | 常量 | 指代真值                                                 |
| `false`      | 常量 | 指代假值                                                 |
| `iota`       | 常量 | 指代常量序列计数起始的`0`                                |
| `nil`        | 零值 | 指代空指针                                               |
| `append`     | 函数 | 用于向一个切片中增加元素                                 |
| `cap`        | 函数 | 返回一个切片的容量（capacity）                           |
| `clear`      | 函数 | 清空 map，重置切片中的全部元素（Go 1.21 以后可用）       |
| `close`      | 函数 | 用于关闭通道                                             |
| `complex`    | 函数 | 用于构建一个复数类型                                     |
| `copy`       | 函数 | 用于复制一个切片                                         |
| `delete`     | 函数 | 用于删除 map 中的键值对和切片中的元素                    |
| `imag`       | 函数 | 获得一个复数的虚数部分                                   |
| `len`        | 函数 | 计算字符串或者切片、数组等可被计数内容的长度（元素数量） |
| `make`       | 函数 | 创建一个切片、Map、结构体的实例                          |
| `max`        | 函数 | 获取给定值中的最大值（Go 1.21 以后可用）                 |
| `min`        | 函数 | 获取给定值中的最小值（Go 1.21 以后可用）                 |
| `new`        | 函数 | 创建一个指定类型的实例并返回其引用                       |
| `panic`      | 函数 | 中断程序运行并抛出异常                                   |
| `println`    | 函数 | 在控制台中输出指定内容                                   |
| `real`       | 函数 | 获得一个复数的实数部分                                   |
| `recover`    | 函数 | 捕获`panic`抛出的异常，并使程序从异常状态中恢复          |

### 空白标识符

空白标识符在之前引入包的章节中就已经见识过了，空白标识符就是一个`_`，它在表达式中充当匿名占位符，在声明表达式、赋值表达式中都具有特殊的含义。

## 关键字

关键字实际上也是标识符，但是是语言中保留的内容，因为其在语言中具有专门的用途和意义，所以不能被用作普通的标识符。Go 中的关键字主要有以下这些。

| 关键字        | 功能                                     |
| ------------- | ---------------------------------------- |
| `break`       | 用于退出循环                             |
| `case`        | 用于定义`switch`语句的分支条件           |
| `chan`        | 用于标识通道类型                         |
| `const`       | 用于定义常量                             |
| `continue`    | 用于提前结束当前的循环，并开始下一次循环 |
| `default`     | 用于定义`switch`和`select`中的默认分支   |
| `defer`       | 用于定义延后执行的语句                   |
| `else`        | 用于定义`if`语句的分支                   |
| `fallthrough` | 用于打通`switch`语句中的多个`case`分支   |
| `for`         | 用于定义循环                             |
| `func`        | 用于定义函数                             |
| `go`          | 用于启动协程                             |
| `goto`        | 用于执行强制跳转                         |
| `if`          | 用于定义条件分支                         |
| `import`      | 用于引入其他的包                         |
| `interface`   | 用于定义接口                             |
| `map`         | 用于定义 map 类型                        |
| `package`     | 用于定义当前代码文件归属的包             |
| `range`       | 用于定义范围边界                         |
| `return`      | 用于从函数中返回值                       |
| `select`      | 用于监听和响应通道                       |
| `struct`      | 用于定义结构体                           |
| `switch`      | 用于定义复杂分支结构                     |
| `type`        | 用于定义类型                             |
| `var`         | 用于定义变量                             |
