# Go 基本语法

任何一门编程语言都是由一些基本的词汇元素组成的，Go 语言也不例外。构成 Go 语言程序代码的基本元素，归类起来主要是标识符、关键字、操作符、标点符号和文字五类。其中所有的空白字符包括空格、Tab 制表符、回车等，都会在程序编译时被忽略，它们只是程序代码中基本元素的分隔符，而不是代码的基本元素之一。

## 分号

在在大多数语言中，分号都是用来作为一个语句结束的标记的，在 Go 语言中也不例外，但是 Go 提供了一些独特的解析规则使得我们在大部分情况下可以省略分号的书写。

1. 当一行的内容被分解为 Token 时，最后一个 Token 是以下内容的时候，分号会被自动插入。
   - 一个标识符；
   - 一个整型、浮点型、复数类型、字符或者是字符串的字面量；
   - `break`、`continue`、`fallthrough`或者是`return`中的一个；
   - `++`、`--`、`)`、`]`或者是`}`中的一个。
1. 一个在`)`或者`}`之前的独占一行的复杂语句。

基于这两条规则，绝大多数语句结尾的分号都可以被省略掉，读者可以在实际编程练习中仔细体会一下。

## 代码文档字符集

Go 语言的代码文档是采用 UTF-8 编码格式的，所以在代码文档中基本上可以使用任意合法的 UTF-8 字符。但是 Go 语言在其 EBNF 格式的定义中，还是将代码文档中可以使用的字符划分成了以下几类：

- 换行符，在 EBNF 定义中使用`newline`标识。
- UTF-8 字符，是指所有除了换行符以外的 UTF-8 码位，在 EBNF 定义中使用`unicode_char`标识。
- UTF-8 字母，是指 UTF-8 码位中，所有被归类为字母的码位，在 EBNF 定义中使用`unicode_letter`标识。
- UTF-8 数字，是指 UTF-8 码位中，所有被归类为数字或者数字位的码位，在 EBNF 定义中使用`unicode_digit`标识。

通过以上对于代码文档中可以使用的字符的定义，再结合 Go 语言代码中出现的内容，就可以定义出也以下基本字符和数字。

```
letter = unicode_letter | "_" .
decimal_digit = "0" ... "9" .
binary_digit = "0" | "1" .
octal_digit = "0" ... "7" .
hex_digit = "0" ... "9" | "A" ... "F" | "a" ... "f" .
```

这套定义也就是说，在 Go 语言代码中，所有的合法字母除了包括 UTF-8 中被归类为字母的码位以外，还包括下划线`_`。剩下的四个定义分别对应十进制数字、二进制数字、八进制数字和十六进制数字。

```admonish tip
这些基础的EBNF定义虽然简单，但是它是构成后文中其他复杂EBNF定义的基础。
```
